import subprocess
import tempfile
from typing import Optional

from .pip import read_requirements_file
from .types import TypePath


def write_requirements_file_from_project_dir(
    project_dir: TypePath,
    out_path: TypePath,
    extra_args: Optional[list[str]] = None,
) -> None:
    command = [
        "uv",
        "export",
        "--project", str(project_dir),
        "--no-emit-project",
        "--no-dev",
        "--no-hashes",
        "--quiet",
        "--output-file", str(out_path),
    ]
    if extra_args is not None:
        command.extend(extra_args)
    try:
        subprocess.run(command, check=True)
    except subprocess.CalledProcessError as e:
        msg = f"Failed to write requirements file from uv project: {e}"
        raise RuntimeError(msg) from e


def get_requirents_from_project_dir(
    project_dir: TypePath,
    uv_args: Optional[list[str]] = None,
) -> list[str]:
    with tempfile.NamedTemporaryFile(mode="w") as f:
        write_requirements_file_from_project_dir(
            project_dir,
            f.name,
            extra_args=uv_args,
        )
        return read_requirements_file(f.name)



from .types import TypePath


def read_requirements_file(requirements_file: TypePath) -> list[str]:
    """Read a requirements file and return a list of requirements.

    Removes comments and empty lines.

    Args:
        requirements_file: Path to the requirements file.
    """
    with open(requirements_file) as f:
        lines = f.readlines()
    requirements = []
    for line in lines:
        line = line.strip()
        if not line or line.startswith("#"):
            continue
        requirements.append(line)
    return requirements



from pathlib import Path

import typer

from .conda import make_conda_env_from_project_dir

app = typer.Typer()


@app.command(
    no_args_is_help=True,
    context_settings={
        "allow_extra_args": True,
        "ignore_unknown_options": True,
    },
)
def uv2conda(
    input_project_dir: Path = typer.Option(
        ...,
        "--input-project-dir",
        "-i",
        file_okay=False,
        dir_okay=True,
        exists=True,
        readable=True,
        help="Path to the project directory",
    ),
    name: str = typer.Option(
        ...,
        "--name",
        "-n",
        help="Name of the conda environment",
    ),
    python_version: str = typer.Option(
        ...,
        "--python",
        "-p",
        help="Python version",
    ),
    out_conda_path = typer.Option(
        ...,
        "--out-conda-path",
        "-o",
        file_okay=True,
        dir_okay=False,
        writable=True,
        help="Path to the output conda environment file",
    ),
    context: typer.Context = typer.Option(None),
):
    make_conda_env_from_project_dir(
        input_project_dir,
        name=name,
        python_version=python_version,
        out_path=out_conda_path,
        uv_args=context.args,
    )



from typing import Optional
from typing import Union

import yaml

from .pip import read_requirements_file
from .python import is_valid_python_version
from .types import TypePath
from .uv import get_requirents_from_project_dir

TypePipEnv = dict[str, list[str]]
TypeCondaDependency = Union[str, TypePipEnv]
TypeChannels = list[str]
TypeCondaEnv = dict[str, Union[str, TypeChannels, list[TypeCondaDependency]]]


def make_conda_env_from_dependencies(
    name: str,
    python_version: str,
    channels: Optional[list[str]] = None,
    conda_dependencies: Optional[list[str]] = None,
    pip_dependencies: Optional[list[str]] = None,
    out_path: Optional[TypePath] = None,
    return_yaml: bool = False,
) -> TypeCondaEnv | str:
    if not is_valid_python_version(python_version):
        raise ValueError(f'Invalid Python version: "{python_version}"')
    env: TypeCondaEnv = {
        "name": name,
    }
    if channels is not None:
        env["channels"] = channels
    if conda_dependencies is not None or pip_dependencies is not None:
        env["dependencies"] = [
            f"python={python_version}",
        ]
        if conda_dependencies:
            env["dependencies"].extend(conda_dependencies)
        if pip_dependencies:
            env["dependencies"].append("pip")
            env["dependencies"].append({"pip": pip_dependencies})

    if return_yaml or out_path is not None:
        yaml_string = yaml.dump(env, sort_keys=False)
        if out_path is not None:
            with open(out_path, "w") as f:
                f.write(yaml_string)
        return yaml_string

    return env


def make_conda_env_from_requirements_file(
    requirements_path: TypePath,
    *args,
    **kwargs,
) -> TypeCondaEnv | str:
    return make_conda_env_from_dependencies(
        *args,
        **kwargs,
        pip_dependencies=read_requirements_file(requirements_path),
    )


def make_conda_env_from_project_dir(
    project_dir: TypePath,
    *args,
    **kwargs,
) -> TypeCondaEnv | str:
    pip_requirements = get_requirents_from_project_dir(
        project_dir,
        uv_args=kwargs.pop("uv_args", None),
    )
    return make_conda_env_from_dependencies(
        *args,
        **kwargs,
        pip_dependencies=pip_requirements,
    )



from .conda import make_conda_env_from_dependencies
from .conda import make_conda_env_from_requirements_file

__all__ = [
    "make_conda_env_from_dependencies",
    "make_conda_env_from_requirements_file",
]
